@@ -7,6 +7,7 @@ import hypothesis.strategies as st
 from hypothesis import given, assume, example
 
 import astropy.units as u
+from drms import DrmsQueryError
 
 from sunpy.net import attr
 from sunpy.net.vso import attrs as va
@@ -90,9 +91,25 @@ def check_response(query, unifiedresp):
 
 @pytest.mark.remote_data
 def test_save_path():
+    qr = Fido.search(a.Instrument('EVE'), a.Time("2016/10/01", "2016/10/02"), a.Level(0))
+
+    # Test when path is str
     with tempfile.TemporaryDirectory() as target_dir:
-        qr = Fido.search(a.Instrument('EVE'), a.Time("2016/10/01", "2016/10/02"), a.Level(0))
-        files = Fido.fetch(qr, path=os.path.join(target_dir, "{instrument}"+os.path.sep+"{level}"))
+        files = Fido.fetch(qr, path=os.path.join(target_dir, "{instrument}", "{level}"))
+        for f in files:
+            assert target_dir in f
+            assert "eve{}0".format(os.path.sep) in f
+
+
+@pytest.mark.remote_data
+def test_save_path_pathlib():
+    pathlib = pytest.importorskip('pathlib')
+    qr = Fido.search(a.Instrument('EVE'), a.Time("2016/10/01", "2016/10/02"), a.Level(0))
+
+    # Test when path is pathlib.Path
+    with tempfile.TemporaryDirectory() as target_dir:
+        path = pathlib.Path(target_dir, "{instrument}", "{level}")
+        files = Fido.fetch(qr, path=path)
         for f in files:
             assert target_dir in f
             assert "eve{}0".format(os.path.sep) in f
@@ -127,9 +144,10 @@ def test_no_time_error():
     assert all(str(a) not in str(excinfo.value) for a in query2.attrs)
 
 
+@pytest.mark.remote_data
 def test_no_match():
-    with pytest.raises(NoMatchError):
-        Fido.search(a.Time("2016/10/01", "2016/10/02"), a.jsoc.Series("bob"),
+    with pytest.raises(DrmsQueryError):
+        Fido.search(a.jsoc.Time("2016/10/01", "2016/10/02"), a.jsoc.Series("bob"),
                     a.vso.Sample(10*u.s))
 
 
