@@ -267,7 +267,7 @@ class TimeFreq(object):
         return ret
 
 
-@deprecated("0.8", "this module is being moved to sunpyspectra - https://github.com/sunpy/spectra")
+@deprecated("0.8", "this module is being moved to radiospectra - http://docs.sunpy.org/projects/radiospectra/")
 class Spectrogram(Parent):
     """
     Spectrogram Class.
@@ -887,7 +887,7 @@ class Spectrogram(Parent):
         return format_coord
 
 
-@deprecated("0.8", "this module is being moved to sunpyspectra - https://github.com/sunpy/spectra")
+@deprecated("0.8", "this module is being moved to radiospectra - http://docs.sunpy.org/projects/radiospectra/")
 class LinearTimeSpectrogram(Spectrogram):
     """
     Spectrogram evenly sampled in time.
