@@ -38,6 +38,9 @@ class SunPyTest(AstropyTest):
         # Run tests that check figure generation
         ('figure', None,
          'Run tests that compare figures against stored hashes.'),
+        # Run only tests that check figure generation
+        ('figure-only', None,
+         'Only run tests that compare figures against stored hashes.'),
         # Calculate test coverage
         ('coverage', 'c',
          'Create a coverage report. Requires the coverage package.'),
@@ -54,18 +57,17 @@ class SunPyTest(AstropyTest):
     ]
 
     user_options = _fix_user_options(user_options)
-
     package_name = ''
 
     def initialize_options(self):
         self.package = ''
-        #self.test_path = None
         self.verbose_results = False
         self.plugins = None
         self.args = None
         self.online = False
         self.online_only = False
         self.figure = False
+        self.figure_only = False
         self.coverage = False
         self.cov_report = 'term' if self.coverage else None
         self.docs_path = os.path.abspath('docs')
@@ -89,6 +91,8 @@ class SunPyTest(AstropyTest):
         Build a Python script to run the tests.
         """
 
+        self._validate_required_deps()  # checks if modules are installed
+
         cmd_pre = ''  # Commands to run before the test function
         cmd_post = ''  # Commands to run after the test function
 
@@ -99,6 +103,8 @@ class SunPyTest(AstropyTest):
 
         online = self.online
         offline = not self.online_only
+        figure = self.figure
+        figure_only = self.figure_only
 
         cmd = ('{cmd_pre}{0}; import {1.package_name}, sys; result = ('
                '{1.package_name}.self_test('
@@ -109,6 +115,7 @@ class SunPyTest(AstropyTest):
                'online={online!r}, '
                'offline={offline!r}, '
                'figure={figure!r}, '
+               'figure_only={figure_only!r}, '
                'cov_report={1.cov_report!r})); '
                '{cmd_post}'
                'sys.exit(result)')
@@ -116,7 +123,8 @@ class SunPyTest(AstropyTest):
                        self,
                        online=online,
                        offline=offline,
-                       figure=self.figure,
+                       figure=figure,
+                       figure_only=figure_only,
                        cmd_pre=cmd_pre,
                        cmd_post=cmd_post)
         return x
