@@ -3,6 +3,7 @@
 import abc
 
 import numpy as np
+import matplotlib as mpl
 import matplotlib.pyplot as plt
 import matplotlib.widgets as widgets
 import matplotlib.animation as mplanim
@@ -525,6 +526,21 @@ class ArrayAnimator(BaseFuncAnimator):
             raise ValueError("spatial_axes and slider_axes mismatch")
 
         self.axis_ranges = self._sanitize_axis_ranges(axis_ranges, data)
+        # Due to some reason, if a slider axis range is of length two and the
+        # difference between the entries is 1, the slider start and end both get
+        # set to the 0th value stopping the animation updating the plot.
+        # In this case iterate the latter element by one to get the desired behaviour.
+        hacked_axis_ranges = [False] * len(self.axis_ranges)
+        index_changed = [None] * len(self.axis_ranges)
+        for i in range(len(self.axis_ranges)):
+            if len(self.axis_ranges[i]) == 2 and data.shape[i] == 2:
+                if 0 <= (self.axis_ranges[i][-1] - self.axis_ranges[i][0]) <= 1:
+                    axis_ranges[i][-1] += 1
+                    index_changed[i] = -1
+                elif -1 <= (self.axis_ranges[i][-1] - self.axis_ranges[i][0]) <= 0:
+                    axis_ranges[i][0] -= 1
+                    index_changed[i] = 0
+                hacked_axis_ranges[i] = True
 
         # create data slice
         self.frame_slice = [slice(None)] * self.naxis
@@ -536,6 +552,15 @@ class ArrayAnimator(BaseFuncAnimator):
         base_kwargs.update(kwargs)
         BaseFuncAnimator.__init__(self, data, **base_kwargs)
 
+        # Undo hack of length 2 axis ranges
+        for i in np.arange(len(self.axis_ranges)):
+            if len(self.axis_ranges[i]) == 2 and data.shape[i] == 2:
+                if hacked_axis_ranges[i]:
+                    if index_changed[i] == 0:
+                        axis_ranges[i][0] += 1
+                    elif index_changed[i] == -1:
+                        axis_ranges[i][-1] -= 1
+
     @property
     def frame_index(self):
         """
@@ -580,7 +605,7 @@ class ArrayAnimator(BaseFuncAnimator):
         """
         # If no axis range at all make it all [min,max] pairs
         if axis_ranges is None:
-            axis_ranges = [[0, i] for i in data.shape]
+            axis_ranges = [None] * data.ndim
 
         # need the same number of axis ranges as axes
         if len(axis_ranges) != data.ndim:
@@ -589,20 +614,20 @@ class ArrayAnimator(BaseFuncAnimator):
         # For each axis validate and translate the axis_ranges
         for i, d in enumerate(data.shape):
             # If [min,max] pair or None
-            if axis_ranges[i] is None or len(axis_ranges[i]) == 2:
+            if axis_ranges[i] is None or (len(axis_ranges[i]) == 2 and d > 2):
                 # If min==max or None
                 if axis_ranges[i] is None or axis_ranges[i][0] == axis_ranges[i][1]:
                     if i in self.slider_axes:
-                        axis_ranges[i] = np.linspace(0, d, d)
+                        axis_ranges[i] = np.arange(0, d+1)
                     else:
                         axis_ranges[i] = [0, d]
                         # min max pair for slider axes should be converted
                         # to an array
                 elif i in self.slider_axes:
-                    axis_ranges[i] = np.linspace(axis_ranges[i][0], axis_ranges[i][1], d)
+                    axis_ranges[i] = np.arange(axis_ranges[i][0], axis_ranges[i][1]+1)
 
             # If we have a whole list of values for the axis, make sure we are a slider axis.
-            elif len(axis_ranges[i]) == d:
+            elif len(axis_ranges[i]) == d or axis_ranges[i].shape == data.shape:
                 axis_ranges[i] = np.array(axis_ranges[i])
 
             # If above criteria are not met, raise an error.
@@ -610,6 +635,7 @@ class ArrayAnimator(BaseFuncAnimator):
                 raise ValueError("axis_ranges must be None or a list with length equal to number "
                                  "of axes in data whose elements are either None, [min,max], "
                                  "or a list/array of same length as the plot/image axis of data.")
+
         return axis_ranges
 
     @abc.abstractmethod
@@ -690,6 +716,10 @@ class ImageAnimator(ArrayAnimator):
         # Define number of slider axes.
         self.naxis = data.ndim
         self.num_sliders = self.naxis-2
+        # Define marker to determine if plot axes values are supplied via array of
+        # pixel values or min max pair. This will determine the type of image produced
+        # and hence how to plot and update it.
+        self._non_regular_plot_axis = False
         # Run init for parent class
         super(ImageAnimator, self).__init__(data, image_axes=image_axes,
                                             axis_ranges=axis_ranges, **kwargs)
@@ -700,15 +730,43 @@ class ImageAnimator(ArrayAnimator):
         extent = []
         # reverse because numpy is in y-x and extent is x-y
         for i in self.image_axes[::-1]:
-            extent += self.axis_ranges[i]
+            if self._non_regular_plot_axis is False and len(self.axis_ranges[i]) > 2:
+                self._non_regular_plot_axis = True
+            extent.append(self.axis_ranges[i][0])
+            extent.append(self.axis_ranges[i][-1])
 
         imshow_args = {'interpolation': 'nearest',
                        'origin': 'lower',
-                       'extent': extent,
-                       }
-
+                       'extent': extent}
         imshow_args.update(self.imshow_kwargs)
-        im = ax.imshow(self.data[self.frame_index], **imshow_args)
+
+        # If value along an axis is set with an array, generate a NonUniformImage
+        if self._non_regular_plot_axis:
+            # If user has inverted the axes, transpose the data so the dimensions match.
+            if self.image_axes[0] < self.image_axes[1]:
+                data = self.data[self.frame_index].transpose()
+            else:
+                data = self.data[self.frame_index]
+            # Initialize a NonUniformImage with the relevant data and axis values and
+            # add the image to the axes.
+            im = mpl.image.NonUniformImage(ax, **imshow_args)
+            im.set_data(self.axis_ranges[self.image_axes[0]],
+                        self.axis_ranges[self.image_axes[1]], data)
+            ax.add_image(im)
+            # Define the xlim and ylim bearing in mind that the pixel values along
+            # the axes are plotted in the middle of the pixel.  Therefore make sure
+            # there's half a pixel buffer either side of the ends of the axis ranges.
+            x_ranges = self.axis_ranges[self.image_axes[0]]
+            xlim = (x_ranges[0] - (x_ranges[1] - x_ranges[0]) / 2.,
+                    x_ranges[-1] + (x_ranges[-1] - x_ranges[-2]) / 2.)
+            y_ranges = self.axis_ranges[self.image_axes[1]]
+            ylim = (y_ranges[0] - (y_ranges[1] - y_ranges[0]) / 2.,
+                    y_ranges[-1] + (y_ranges[-1] - y_ranges[-2]) / 2.)
+            ax.set_xlim(xlim)
+            ax.set_ylim(ylim)
+        else:
+            # Else produce a more basic plot with regular axes.
+            im = ax.imshow(self.data[self.frame_index], **imshow_args)
         if self.if_colorbar:
             self._add_colorbar(im)
 
@@ -721,40 +779,17 @@ class ImageAnimator(ArrayAnimator):
         ind = np.argmin(np.abs(self.axis_ranges[ax_ind] - val))
         self.frame_slice[ax_ind] = ind
         if val != slider.cval:
-            im.set_array(self.data[self.frame_index])
+            if self._non_regular_plot_axis:
+                if self.image_axes[0] < self.image_axes[1]:
+                    data = self.data[self.frame_index].transpose()
+                else:
+                    data = self.data[self.frame_index]
+                im.set_data(self.axis_ranges[self.image_axes[0]],
+                            self.axis_ranges[self.image_axes[1]], data)
+            else:
+                im.set_array(self.data[self.frame_index])
             slider.cval = val
 
-    def _sanitize_axis_ranges(self, axis_ranges, data):
-        """
-        This method takes the various allowed values of axis_ranges and returns
-        them in a standardized way for the rest of the class to use.
-
-        The outputted axis_ranges describe the physical coordinates of the
-        array axes.
-
-        The allowed values of axis_range is either None or a list.
-        If axis_ranges is None then all axes are assumed to be not scaled and
-        use array indices.
-
-        Where axis_ranges is a list it must have the same length as the number
-        of axis as the array and each element must be one of the following:
-
-            * None: Build a min,max pair or linspace array of array indices
-            * [min, max]: leave for image axes or convert to a array for slider axes
-            (from min to max in axis length steps)
-            * [min, max] pair where min == max: convert to array indies min,max pair or array.
-            * array of axis length, check that it was passed for a slider axes and do nothing
-            if it was, error if it is not.
-        """
-        # Run super class's version of this function.
-        axis_ranges = super(ImageAnimator, self)._sanitize_axis_ranges(axis_ranges, data)
-        # Check that axis ranges which are array type are only for slider axes.
-        for i, d in enumerate(data.shape):
-            if len(axis_ranges[i]) == d:
-                if i not in self.slider_axes:
-                    raise ValueError("Slider axes mis-match, non-slider axes need [min,max] pairs")
-        return axis_ranges
-
 
 class LineAnimator(ArrayAnimator):
     """
@@ -783,18 +818,11 @@ class LineAnimator(ArrayAnimator):
     fig: `matplotlib.figure`
         Figure to use
 
-    axis_ranges: `list` of physical coordinates for array or None
-        If None array indices will be used for all axes.
-        X-axis values must be supplied (if desired) as an array in the
-        element of axis_ranges corresponding to the plot_axis_index in the
-        data input arg, i.e. xdata = axis_ranges[plot_axis_index].
-        Also, the number of x-axis values must correspond to the number
-        of y-axis values, i.e. len(axis_ranges[plot_axis_index]) must equal
-        len(data[plot_axis_index]).
-        For the slider axes a [min, max] pair can be specified or an array the
-        same length as the axis which will provide all values for that slider.
-        If None is specified for an axis then the array indices will be used
-        for that axis.
+    axis_ranges: `list` of physical coordinates or None
+        Each element of axis_ranges provides an array of physical coordinates for the
+        corresponding dimension of the data array.  If an element is None, array indices will be
+        used for that axis.  If axis_range itself is None, array indices will be used for all axes.
+        For more information, see the Notes section of this docstring.
 
     xlabel: `str`
         Label of x-axis of plot.
@@ -819,34 +847,61 @@ class LineAnimator(ArrayAnimator):
 
     Extra keywords are passed to plot.
 
+    Notes
+    -----
+    Additional information on API of axes_ranges kwarg.
+
+    #. X-axis values must be supplied (if desired) as an array in the element of
+       the axis_ranges list corresponding to the plot_axis_index in the data array, i.e.
+       ``x_axis_values == axis_ranges[plot_axis_index]``
+
+    #. The shape of the x-axis values array can take two forms.
+
+       a) First, it can equal the the length of the data array along the dimension
+          corresponding to the x-axis, i.e.
+          ``len(axis_ranges[plot_axis_index]) == len(data[plot_axis_index])``
+          In this scenario the same x-axis values are used in every frame of the animation.
+       b) Second, the x-axis array can have the same shape as the data array.
+          In this scenario the x-axis is refreshed for each frame. For example, if
+          ``data.shape == axis_ranges[plot_axis_index] == (4, 3)``,
+          where ``plot_axis_index == 0``, the 0th frame of the animation will show data from
+          ``data[:, 0]`` with the x-axis described by ``axis_ranges[plot_axis_index][:, 0]``,
+          while the 1st frame will show data from ``data[:, 1]`` with the x-axis described by
+          ``axis_ranges[plot_axis_index][:, 1]``.
+
+    #. For the slider axes the axis range is an array of the same length as the dimension of the
+       data array to which that slider corresponds.
+
     """
 
     def __init__(self, data, plot_axis_index=-1, axis_ranges=None, ylabel=None, xlabel=None,
                  xlim=None, ylim=None, **kwargs):
         # Check inputs.
-        plot_axis_index = int(plot_axis_index)
-        if plot_axis_index not in range(-2, 2):
-            raise ValueError("plot_axis_index must be either 0 or 1 (or equivalent "
-                             "negative indices) referring to the axis of data to be "
-                             "used for a single plot.")
+        self.plot_axis_index = int(plot_axis_index)
+        if self.plot_axis_index not in range(-data.ndim, data.ndim):
+            raise ValueError("plot_axis_index must be within range of number of data dimensions"
+                             " (or equivalent negative indices).")
         if data.ndim < 2:
             raise ValueError("data must have at least two dimensions.  One for data "
                              "for each single plot and at least one for time/iteration.")
         # Ensure axis_ranges are input correctly.
-        if axis_ranges[plot_axis_index] is not None:
-            if len(axis_ranges[plot_axis_index]) != data.shape[plot_axis_index]:
-                raise ValueError("The plot_axis_index axis range must be specified as None "
-                                 "or an array. Not [min, max]")
+        if axis_ranges is not None:
+            if axis_ranges[self.plot_axis_index] is not None:
+                if (len(axis_ranges[self.plot_axis_index]) != data.shape[self.plot_axis_index] and
+                        axis_ranges[self.plot_axis_index].shape != data.shape):
+                    raise ValueError("The plot_axis_index axis range must be specified as None "
+                                     "a 1D array of same length as plot_axis_index axis, "
+                                     "or an array of same shape as data.")
         # Define number of slider axes.
         self.naxis = data.ndim
         self.num_sliders = self.naxis-1
         # Attach data to class.
         if axis_ranges is not None and all(axis_range is None for axis_range in axis_ranges):
             axis_ranges = None
-        if axis_ranges[plot_axis_index] is None:
-            self.xdata = np.arange(data.shape[plot_axis_index])
+        if axis_ranges is None or axis_ranges[self.plot_axis_index] is None:
+            self.xdata = np.arange(data.shape[self.plot_axis_index])
         else:
-            self.xdata = np.asarray(axis_ranges[plot_axis_index])
+            self.xdata = np.asarray(axis_ranges[self.plot_axis_index])
         if ylim is None:
             ylim = (data.min(), data.max())
         if xlim is None:
@@ -856,7 +911,7 @@ class LineAnimator(ArrayAnimator):
         self.xlabel = xlabel
         self.ylabel = ylabel
         # Run init for base class
-        super(LineAnimator, self).__init__(data, image_axes=[plot_axis_index],
+        super(LineAnimator, self).__init__(data, image_axes=[self.plot_axis_index],
                                            axis_ranges=axis_ranges, **kwargs)
 
     def plot_start_image(self, ax):
@@ -869,17 +924,32 @@ class LineAnimator(ArrayAnimator):
             ax.set_ylabel(self.ylabel)
         plot_args = {}
         plot_args.update(self.imshow_kwargs)
-        line, = ax.plot(self.xdata, self.data[self.frame_index], **plot_args)
+        if self.xdata.shape == self.data.shape:
+            item = [0] * self.data.ndim
+            item[self.plot_axis_index] = slice(None)
+            xdata = np.squeeze(self.xdata[tuple(item)])
+        else:
+            xdata = self.xdata
+        line, = ax.plot(xdata, self.data[self.frame_index], **plot_args)
         return line
 
     def update_plot(self, val, line, slider):
         """Updates plot based on slider/array dimension being iterated."""
         val = int(val)
         ax_ind = self.slider_axes[slider.slider_ind]
-        ind = np.argmin(np.abs(self.axis_ranges[ax_ind] - val))
+        ind = int(np.argmin(np.abs(self.axis_ranges[ax_ind] - val)))
         self.frame_slice[ax_ind] = ind
         if val != slider.cval:
             line.set_ydata(self.data[self.frame_index])
+            if self.xdata.shape == self.data.shape:
+                item = [int(slid._slider.val) for slid in self.sliders]
+                item[ax_ind] = val
+                if self.plot_axis_index < 0:
+                    i = self.data.ndim + self.plot_axis_index
+                else:
+                    i = self.plot_axis_index
+                item.insert(i, slice(None))
+                line.set_xdata(self.xdata[tuple(item)])
             slider.cval = val
 
 
